@@ -4,9 +4,10 @@ from .utils import (
     config,
     VarnameRetrievingError,
     QualnameNonUniqueError,
+    NonVariableArgumentError,
     MultiTargetAssignmentWarning,
     MaybeDecoratedFunctionWarning
 )
-from .core import varname, nameof, will
+from .core import varname, nameof, will, argname
 
-__version__ = "0.6.0"
+__version__ = "0.6.1"
