@@ -12,11 +12,12 @@ Attributes:
 import sys
 import dis
 import ast
+import warnings
 import inspect
 from os import path
 from functools import lru_cache
 from types import ModuleType, FunctionType, CodeType, FrameType
-from typing import Optional, Tuple, Union, List
+from typing import Optional, Tuple, Union, List, MutableMapping, Callable
 
 from executing import Source
 
@@ -48,6 +49,10 @@ class QualnameNonUniqueError(Exception):
     """When a qualified name is used as an ignore element but references to
     multiple objects in a module"""
 
+class NonVariableArgumentError(Exception):
+    """When vars_only is True but try to retrieve name of
+    a non-variable argument"""
+
 class MaybeDecoratedFunctionWarning(Warning):
     """When a suspecious decorated function used as ignore function directly"""
 
@@ -63,7 +68,8 @@ def cached_getmodule(codeobj: CodeType):
 def get_node(
         frame: int,
         ignore: Optional[IgnoreType] = None,
-        raise_exc: bool = True
+        raise_exc: bool = True,
+        ignore_lambda: bool = True
 ) -> Optional[ast.AST]:
     """Try to get node from the executing object.
 
@@ -74,12 +80,19 @@ def get_node(
     When the node can not be retrieved, try to return the first statement.
     """
     from .ignore import IgnoreList
-    ignore = IgnoreList.create(ignore)
+    ignore = IgnoreList.create(ignore, ignore_lambda=ignore_lambda)
     try:
         frame = ignore.get_frame(frame)
     except VarnameRetrievingError:
         return None
 
+    return get_node_by_frame(frame, raise_exc)
+
+def get_node_by_frame(
+        frame: FrameType,
+        raise_exc: bool = True
+) -> Optional[ast.AST]:
+    """Get the node by frame, raise errors if possible"""
     exect = Source.executing(frame)
 
     if exect.node:
@@ -123,7 +136,7 @@ def node_name(node: ast.AST) -> Optional[Union[str, Tuple[Union[str, tuple]]]]:
 def bytecode_nameof(frame: int = 1) -> str:
     """Bytecode version of nameof as a fallback"""
     from .ignore import IgnoreList
-    frame = IgnoreList.create(None).get_frame(frame)
+    frame = IgnoreList.create().get_frame(frame)
     return _bytecode_nameof_cached(frame.f_code, frame.f_lasti)
 
 @lru_cache()
@@ -225,3 +238,122 @@ def debug_ignore_frame(
         msg = (f'{msg} [In {frameinfo.function!r} at '
                f'{frameinfo.filename}:{frameinfo.lineno}]')
     sys.stderr.write(f'[{__name__}] DEBUG: {msg}\n')
+
+def argnode_source(source: Source, node: ast.AST, vars_only: bool) -> str:
+    """Get the source of an argument node"""
+    if vars_only:
+        if not isinstance(node, ast.Name):
+            raise NonVariableArgumentError(
+                f'Argument {ast.dump(node)} is not a variable.'
+            )
+        return node.id
+    return source.asttokens().get_text(node)
+
+def get_argument_sources(
+        frame: FrameType,
+        node: ast.Call,
+        func: Callable,
+        vars_only: bool = True
+) -> MutableMapping[str, str]:
+    """Get the sources for argument from an ast.Call node
+
+    >>> def func(a, b, c, d=4):
+    >>>  ...
+    >>> x = y = z = 1
+    >>> func(y, x, c=z)
+    >>> # argument_sources = {'a': 'y', 'b', 'x', 'c': 'z'}
+    """
+    from .ignore import IgnoreList
+    frame = IgnoreList.create(ignore_lambda=False).get_frame(2)
+    # <Signature (a, b, c, d=4)>
+    signature = inspect.signature(func, follow_wrapped=False)
+    # func(y, x, c=z)
+    # ['y', 'x'], {'c': 'z'}
+    source = Source.for_frame(frame)
+    arg_sources = [argnode_source(source, argnode, vars_only)
+                   for argnode in node.args]
+    kwarg_sources = {argnode.arg: argnode_source(source,
+                                                 argnode.value,
+                                                 vars_only)
+                     for argnode in node.keywords}
+    bound_args = signature.bind_partial(*arg_sources, **kwarg_sources)
+    return bound_args.arguments
+
+def get_function_called_argname(
+        frame: FrameType,
+        node: ast.AST
+) -> Callable:
+    """Get the function who called argname"""
+    # We need node to be ast.Call
+    if not isinstance(node, ast.Call):
+        raise VarnameRetrievingError(
+            f"Expect an 'ast.Call' node, but got {type(node)!r}. "
+            "Are you using 'argname' inside a function?"
+        )
+
+    # variable
+    if isinstance(node.func, ast.Name):
+        func = frame.f_locals.get(
+            node.func.id,
+            frame.f_globals.get(node.func.id)
+        )
+        if func is None: # pragma: no cover
+            # not sure how it would happen but in case
+            raise VarnameRetrievingError(
+                f"Cannot retrieve the function by {node.func.id!r}."
+            )
+        return func
+
+    # use pure_eval
+    pure_eval_fail_msg = None
+    try:
+        from pure_eval import Evaluator, CannotEval
+    except ImportError:
+        pure_eval_fail_msg = "'pure_eval' is not installed."
+    else:
+        try:
+            evaluator = Evaluator.from_frame(frame)
+            return evaluator[node.func]
+        except CannotEval:
+            pure_eval_fail_msg = (
+                f"Cannot evaluate node {ast.dump(node.func)} "
+                "using 'pure_eval'."
+            )
+
+    # try eval
+    warnings.warn(
+        f"{pure_eval_fail_msg} "
+        "Using 'eval' to get the function that calls 'argname'. "
+        "Try calling it using a variable reference to the function, or "
+        "passing the function to 'argname' explicitly."
+    )
+    expr = ast.Expression(node.func)
+    code = compile(expr, '<ast-call>', 'eval')
+    # pylint: disable=eval-used
+    return eval(code, frame.f_globals, frame.f_locals)
+
+def parse_argname_subscript(node: ast.Subscript):
+    """Parse the ast.Subscript node passed to argname
+
+    Make sure:
+    1. node.value is an ast.Name node
+    2. node.slice is a ast.Constant node
+
+    This is separated as a function, since we want to see in the future whether
+    non-ast.Constant node can be supported as node.slice.
+    """
+    name = node.value
+    if not isinstance(name, ast.Name):
+        raise ValueError(f"Expect {ast.dump(name)} to be a variable.")
+
+    subscript = node.slice
+    if isinstance(subscript, ast.Index):
+        subscript = subscript.value
+    if not isinstance(subscript, (ast.Str, ast.Num, ast.Constant)):
+        raise ValueError(f"Expect {ast.dump(subscript)} to be a constant.")
+
+    subscript = getattr(subscript, 'value', subscript) # ast.Index, ast.Constant
+    subscript = getattr(subscript, 'n', subscript) # ast.Num
+    subscript = getattr(subscript, 's', subscript) # ast.Str
+
+    return name.id, subscript
